<?php
/*
 * @package    Nevigen Installer Plugin
 * @version    1.2.0
 * @author     Nevigen.com - https://nevigen.com
 * @copyright  Copyright © Nevigen.com. All rights reserved.
 * @license    Proprietary. Copyrighted Commercial Software
 * @link       https://nevigen.com
 */

namespace Joomla\Plugin\Installer\Nevigen\Extension;

\defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Http\Http;
use Joomla\CMS\Installer\Installer;
use Joomla\CMS\Installer\InstallerHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Table\Extension;
use Joomla\Database\DatabaseAwareTrait;
use Joomla\Registry\Registry;

class Nevigen extends CMSPlugin
{
	use DatabaseAwareTrait;

	/**
	 * Affects constructor behavior.
	 *
	 * @var  boolean
	 *
	 * @since  1.1.0
	 */
	protected $autoloadLanguage = true;

	/**
	 * List extensions.
	 *
	 * @var    string
	 *
	 * @since  1.1.0
	 */
	protected ?string $list = null;

	/**
	 * Error html.
	 *
	 * @var    string
	 *
	 * @since  1.1.0
	 */
	protected ?string $error = null;

	/**
	 * Url request.
	 *
	 * @var    string
	 *
	 * @since  1.1.0
	 */
	protected string $host = 'https://nevigen.com';

	/**
	 * Request api.
	 *
	 * @var    string
	 *
	 * @since  1.1.0
	 */
	protected string $api = '/nevigen_api/';

	/**
	 * Plugin ajax controller.
	 *
	 * @throws  \Exception
	 *
	 * @return  mixed Action response on success, exception on failure.
	 *
	 * @since  1.1.0
	 */
	public function onAjaxNevigen()
	{
		$app = $this->getApplication();
		if ($app->isClient('site'))
		{
			throw new \Exception('Access is denied', 403);
		}

		if ($this->error === null)
		{
			ob_start();
			include_once PluginHelper::getLayoutPath('installer', 'nevigen', 'error');

			$this->error = ob_get_clean();
		}

		if (!$action = $app->input->get('action'))
		{
			throw new \Exception($this->error, 404);
		}

		if (!method_exists($this, $action))
		{
			throw new \Exception($this->error, 404);
		}
		if ($app->checkToken() === false)
		{
			throw new \Exception($this->error, 404);
		}


		return $this->$action();
	}


	/**
	 * Event listener for the `onInstallerAddInstallationTab` event.
	 *
	 * @return  array  Returns an array with the tab information
	 *
	 * @since   1.1.0
	 */
	public function onInstallerAddInstallationTab()
	{
		// Load language files
		$this->loadLanguage();

		$app  = $this->getApplication();
		$doc  = $app->getDocument();
		$lang = $this->getLanguage();

		$tab = [
			'name'  => 'nevigen',
			'label' => Text::_('PLG_INSTALLER_NEVIGEN_TAB_LABEL'),
		];

		// Load asses
		/** @var \Joomla\CMS\WebAsset\WebAssetRegistry $assetsRegistry */
		$assetsRegistry = $doc->getWebAssetManager()->getRegistry();
		$assetsRegistry->addExtensionRegistryFile('plg_installer_nevigen');

		/** @var \Joomla\CMS\WebAsset\WebAssetManager $assets */
		$assets = $doc->getWebAssetManager();
		$assets->usePreset('installer_nevigen.main');
		$doc->addScriptOptions(
			'plg_installer_nevigen',
			[
				'language'   => base64_encode($lang->getTag()),
				'controller' => 'index.php?option=com_ajax&plugin=nevigen&group=installer&format=json',
			]
		);

		if (empty($this->params->get('key')))
		{
			$id  = PluginHelper::getPlugin($this->_type, $this->_name)->id;
			$url = '/administrator/index.php?option=com_plugins&task=plugin.edit&extension_id=' . $id;
			$app->enqueueMessage(Text::sprintf('PLG_INSTALLER_NEVIGEN_ERROR_KEY', $url), 'warning');
		}
		$session = $app->getSession();
		$message = $session->get('nevigen_install_message');
		if (!empty($message))
		{
			$app->enqueueMessage($message, 'success');

			$session->set('nevigen_install_message', null);
		}

		// Render the input
		ob_start();
		include_once PluginHelper::getLayoutPath('installer', 'nevigen');
		$tab['content'] = ob_get_clean();
		$tab['content'] = '<legend>' . $tab['label'] . '</legend>' . $tab['content'];

		return $tab;
	}

	public function getListExtensions()
	{
		$app            = $this->getApplication();
		$url            = $this->host . '/listExtensions.json';
		$extensionsList = [];
		$language       = $this->getLanguage();
		$html           = '';

		if ($this->list === null)
		{
			$data       = $this->sendApi($url);
			$categories = [];

			if (!empty($data))
			{
				$extensionsList   = $data->toArray();
				$lang             = $language->getTag();
				$extensionsActive = $this->getActiveExtensions();

				$table = new Extension($this->getDatabase());
				foreach ($extensionsList as &$item)
				{
					if (!empty($item['category']) && !in_array($item['category'], $categories))
					{
						$categories[] = $item['category'];
					}

					$element = $item['element'];
					$find    = ['type' => $item['type'], 'element' => $element];
					if ($item['type'] === 'plugin')
					{
						$element         = str_replace('plg_', '', $item['element']);
						$explode         = explode('_', $element);
						$folder          = $explode[0];
						$element         = str_replace($folder . '_', '', $element);
						$find['folder']  = $folder;
						$find['element'] = $element;

					}
					$id           = $table->find($find);
					$item['find'] = $id;
					if (!empty($extensionsActive))
					{
						if (in_array($item['element'], $extensionsActive))
						{
							$item['active'] = true;
						}
					}
					$item['title']       = (!empty($item['language'][$lang]['title'])) ? $item['language'][$lang]['title'] : $item['title'];
					$item['description'] = (!empty($item['language'][$lang]['description'])) ? $item['language'][$lang]['description'] : '';

				}

				if (!empty($extensionsActive))
				{
					usort($extensionsList, function ($a, $b) {
						return strnatcmp($b['active'], $a['active']);
					});
				}

				$nevigenSearchValue   = $app->input->get('nevigen_search','');

				ob_start();
				include_once PluginHelper::getLayoutPath('installer', 'nevigen', 'extensions');

				$html = ob_get_clean();
			}

			if (empty($html)) throw new \Exception($this->error, 404);

			$this->list = $html;
		}

		return $this->list;

	}

	public function installExtension()
	{
		if (empty($key = $this->params->get('key'))) return [];

		$app       = $this->getApplication();
		$extension = $app->input->getString('extension');
		$free      = $app->input->getInt('free', 0);
		if ($free > 0)
		{
			$urlExtension = $this->host . '/download?element=' . $extension;
		}
		else
		{
			$urlApi = $this->host . $this->api . $key . '/install';
			$data   = $this->sendApi($urlApi, ['extension' => $extension]);
			if (!empty($data) && (bool) $data['success'] === true && !empty($data['data']))
			{
				$urlExtension = $data['data'];
			}
			else
			{
				throw new \Exception(Text::_('PLG_INSTALLER_NEVIGEN_ERROR_EXTENSION'), 404);
			}

		}

		if (!empty($urlExtension))
		{
			// Download the package at the URL given
			if (!$p_file = InstallerHelper::downloadPackage($urlExtension))
			{
				throw new \Exception(Text::sprintf('COM_INSTALLER_PACKAGE_DOWNLOAD_FAILED', $urlExtension), 404);
			}

			// Unpack the downloaded package file
			if (!$package = InstallerHelper::unpack(Factory::getConfig()->get('tmp_path') . '/' . $p_file, true))
			{
				throw new \Exception(Text::sprintf('COM_INSTALLER_UNPACK_ERROR', $p_file), 404);

			}

			// Check type
			if (!$package['type'])
			{
				InstallerHelper::cleanupInstall($package['packagefile'], $package['extractdir']);

				return false;
			}

			// Get an installer instance
			$installer = Installer::getInstance();
			$installer->setPath('source', $package['dir']);
			if (!$installer->findManifest())
			{
				InstallerHelper::cleanupInstall($package['packagefile'], $package['extractdir']);

				return false;
			}

			// Install the package
			if (!$installer->install($package['dir']))
			{
				InstallerHelper::cleanupInstall($package['packagefile'], $package['extractdir']);

				return false;
			}

			// CleanUp install
			InstallerHelper::cleanupInstall($package['packagefile'], $package['extractdir']);

			$session = $app->getSession();
			$session->set('nevigen_install_message', Text::sprintf('COM_INSTALLER_INSTALL_SUCCESS', $installer->message));
		}

		return true;
	}

	protected function getActiveExtensions()
	{
		if (empty($key = $this->params->get('key'))) return [];

		$url = $this->host . $this->api . $key . '/extensions';

		$data = $this->sendApi($url, ['active' => true]);

		return $data->get('data', []);
	}

	protected function sendApi($url, $data = [], $headers = [])
	{
		try
		{
			if (!empty($data))
			{
				if (empty($this->params->get('key'))) return false;

				if (!isset($data['domain']))
				{
					$data['domain'] = $this->getApplication()->input->server->get('HTTP_HOST', '');
				}
				$response = (new Http())->post($url, $data, $headers);
			}
			else $response = (new Http())->get($url, $headers);

			return new Registry($response->body);
		}
		catch (\Exception $e)
		{
			return false;
		}
	}
}
